﻿using Esri.ArcGISRuntime.Security;
using System;
using System.Diagnostics;
using System.Security.Cryptography.X509Certificates;
using System.Threading.Tasks;

namespace ArcGISCertificateCrashTest
{
    public class ChallengeHandler : IChallengeHandler
    {
        private bool mCancelledLogin;

        // Change the thumbprint to that of a valid client certificate you have installed that is mapped to a portal user.
        private const string ClientCertificateThumbprint = "56F0F2A64378963993A65B963F4266171C06F87C";

        private const string ClientAuthenticationOid = "1.3.6.1.5.5.7.3.2";

        private readonly MapViewModel mMapViewModel;

        #region Public Methods

        public ChallengeHandler(MapViewModel mapViewModel)
        {
            mMapViewModel = mapViewModel;
        }

        public async Task<Credential> CreateCredentialAsync(CredentialRequestInfo requestInfo)
        {
            Credential credentialToReturn = null;

            mCancelledLogin = false;

            while (credentialToReturn == null && !mCancelledLogin)
            {
                credentialToReturn = CreateCredentialInternal(requestInfo);
            }

            return credentialToReturn;
        }

        #endregion Public Methods

        #region Private Methods

        private Credential CreateCredentialInternal(CredentialRequestInfo requestInfo)
        {
            Credential newCredential = null;

            X509Store store = null;

            try
            {
                store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
                store.Open(OpenFlags.ReadOnly);

                if (mMapViewModel.UseDefaultClientCertificate)
                {
                    newCredential = KnownValidCertificate(store, requestInfo);
                }
                else
                {
                    newCredential = UserSelectedCertificate(store, requestInfo);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Exception: " + ex.Message);
            }
            finally
            {
                store?.Close();
            }

            if (newCredential == null)
            {
                mCancelledLogin = true;
            }

            return newCredential;
        }

        private Credential KnownValidCertificate(X509Store store, CredentialRequestInfo requestInfo)
        {
            Credential credential = null;

            var certificates = store.Certificates.Find(X509FindType.FindByThumbprint, ClientCertificateThumbprint, true);

            if (certificates.Count > 0)
            {
                credential = new CertificateCredential(certificates[0])
                {
                    ServiceUri = requestInfo.ServiceUri
                };
            }

            return credential;
        }

        private Credential UserSelectedCertificate(X509Store store, CredentialRequestInfo requestInfo)
        {
            var certificates = store.Certificates.Find(X509FindType.FindByApplicationPolicy, ClientAuthenticationOid, true);

            var credential = SelectCertificate(requestInfo, certificates);

            return credential;
        }

        private Credential SelectCertificate(CredentialRequestInfo requestInfo, X509Certificate2Collection certificates)
        {
            Credential credential = null;

            var title = "Select certificate to connect to:";

            var selection = X509Certificate2UI.SelectFromCollection(certificates, title, requestInfo.ServiceUri.Host, X509SelectionFlag.SingleSelection);

            if (selection.Count > 0)
            {
                credential = new CertificateCredential(selection[0])
                {
                    ServiceUri = requestInfo.ServiceUri
                };
            }

            return credential;
        }

        #endregion Private Methods
    }
}
