using Esri.ArcGISRuntime.Mapping;
using Esri.ArcGISRuntime.Portal;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Runtime.CompilerServices;
using System.Threading.Tasks;
using System.Windows;
using AuthenticationManager = Esri.ArcGISRuntime.Security.AuthenticationManager;

namespace ArcGISCertificateCrashTest
{
    /// <summary>
    /// Provides map data to an application
    /// </summary>
    public class MapViewModel : INotifyPropertyChanged
    {
        private const string NotLoggedIn = "Not logged in";

        // Change the following uri to connect to your own portal instance supporting PKI
        private static readonly Uri sPortalUri = new Uri("https://ibm-esri-105.esri.ad.hursley.ibm.com/arcgis/sharing/rest");

        private string mPortalUser = NotLoggedIn;
        private bool mCreateEnabled = true;
        private bool mUseDefaultClientCertificate = true;
        private Visibility mProgressVisibility = Visibility.Visible;
        private ObservableCollection<PortalItemHolder> mPortalItemHolders = new ObservableCollection<PortalItemHolder>();

        public event PropertyChangedEventHandler PropertyChanged;

        public string PortalUser
        {
            get { return mPortalUser; }
            set { mPortalUser = value; OnPropertyChanged(); }
        }

        public bool CreateEnabled
        {
            get { return mCreateEnabled; }
            set { mCreateEnabled = value; OnPropertyChanged(); }
        }

        public Visibility ProgressVisibility
        {
            get { return mProgressVisibility; }
            set { mProgressVisibility = value; OnPropertyChanged(); }
        }

        public bool UseDefaultClientCertificate
        {
            get { return mUseDefaultClientCertificate; }
            set { mUseDefaultClientCertificate = value; OnPropertyChanged(); }
        }

        public ObservableCollection<PortalItemHolder> PortalItemHolders
        {
            get { return mPortalItemHolders; }
            set { mPortalItemHolders = value; OnPropertyChanged(); }
        }

        public async Task ConnectToPortal()
        {
            try
            {
                CreateEnabled = false;
                ProgressVisibility = Visibility.Visible;

                PortalUser = NotLoggedIn;

                PortalItemHolders.Clear();

                // Note:- The crashing issue still occurs even with the line below commented out!
                ResetConnections();

                var portal = await ArcGISPortal.CreateAsync(sPortalUri);

                Debug.Assert(portal != null);

                PortalUser = portal.User.FullName;

                await GetPortalItems(portal);

            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
            finally
            {
                CreateEnabled = true;
                ProgressVisibility = Visibility.Hidden;
            }
        }

        public void ResetConnections()
        {
            var credList = AuthenticationManager.Current.Credentials.ToList();

            foreach (var credential in credList)
            {
                AuthenticationManager.Current.RemoveCredential(credential);
            }

            // The code below is obviously a bit dodgy as it's using reflection to try and forcefully close the connecions and reset the client certificate.
            // It's not the cause of the problem however, it's been added in an attempt to fix it but unfortunately the error still happens when you try to switch certificates.
            var servicePoint = ServicePointManager.FindServicePoint(sPortalUri);

            var releaseAllConnectionGroups = servicePoint.GetType().GetMethod("ReleaseAllConnectionGroups", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);

            releaseAllConnectionGroups?.Invoke(servicePoint, null);

            var updateClientCertificate = servicePoint.GetType().GetMethod("UpdateClientCertificate", System.Reflection.BindingFlags.Instance | System.Reflection.BindingFlags.NonPublic);

            updateClientCertificate?.Invoke(servicePoint, new object[] { null });
        }

        public async Task GetPortalItems(ArcGISPortal portal)
        {
            IEnumerable<Basemap> results = await portal.GetBasemapsAsync();

            if (results.Any())
            {
                foreach (var basemap in results)
                {
                    mPortalItemHolders.Add(new PortalItemHolder((PortalItem) basemap.Item));
                }
            }
        }

        protected void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
