﻿using Esri.ArcGISRuntime.Portal;
using Esri.ArcGISRuntime.UI;
using System.ComponentModel;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using System.Windows.Media;

namespace ArcGISCertificateCrashTest
{
    public class PortalItemHolder : INotifyPropertyChanged
    {
        public PortalItem PortalItem { get; }

        private ImageSource mImageSource;

        public PortalItemHolder(PortalItem portalItem)
        {
            PortalItem = portalItem;

            // This probably seems like an anti-pattern as we're calling an async method from a constructor.
            // We're intentionally not awaiting the call however so we immediately return an object
            // and a notification fires to update the UI when the thumbnail loads.
            // This is the way Esri advocate via examples.
            LoadImage();
        }

        public async void LoadImage()
        {
            // This line obviously results in a web request and it's here that the unhandled exception occurs due to a null certificate
            // in the HttpclientHandler's client certificate collection.  Note, when it occurs, it always happens on the first request.
            ThumbnailImageSource = await PortalItem.Thumbnail.ToImageSourceAsync();
            Debug.WriteLine($"Loaded Image for {PortalItem.Url}");
        }

        public ImageSource ThumbnailImageSource
        {
            get { return mImageSource; }

            set
            {
                mImageSource = value;
                OnPropertyChanged();
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
